#!/usr/bin/perl -w
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Baruwa - Web 2.0 MailScanner front-end.
# Copyright (C) 2010-2014  Andrew Colin Kissa <andrew@topdog.za.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

use strict;
use Pod::Usage;
use FileHandle;
use Getopt::Long;

my ( $oldconfig, $newconfig, $comments, $UsedOldValue, $UsedDefaultValue );
my ( $key, $value, $origkey, $origline, %oldsettings, %oldkeys, %oldcomments );
my $help         = 0;
my $keepcomments = 1;
my $ReadOldValue = 0;

GetOptions(
    "oldconfig=s" => \$oldconfig,
    "newconfig=s" => \$newconfig,
    'help|?'      => \$help
) or pod2usage(2);

pod2usage(1) if $help;
unless ( $oldconfig && $newconfig && -f $oldconfig && -f $newconfig ) {
    pod2usage(1);
    exit 1;
}

sub process_oldfile {
    my $oldfh = new FileHandle;
    $oldfh->open($oldconfig)
      or die "Cannot read old manifest file $oldconfig, $!";
    $comments = "";
    while (<$oldfh>) {
        chomp;
        $origline = $_;
        s/#.*$//;
        s/^\s+//;
        s/\s+$//;
        ( $comments .= "$origline\n" ), next if /^$/;

        undef $origkey;
        undef $key;
        undef $value;
        /^(.*?)\s*=/;
        $origkey = $1;
        $origline =~ /=\s*(.*)$/;
        $value = $1;

        if ( defined($origkey) ) {
            $key               = lc($origkey);
            $oldsettings{$key} = $value;
            $oldkeys{$key}     = $origkey;
            $oldcomments{$key} = $comments;
            $comments          = "";
            $ReadOldValue++;
        }
    }
    $oldfh->close();
}

sub process_newfile {
    my $newfh = new FileHandle;
    $newfh->open($newconfig)
      or die "Cannot read new manifest file $newconfig, $!";
    my $defaultvalue;
    $UsedOldValue     = 0;
    $UsedDefaultValue = 0;
    $comments         = "";
    while (<$newfh>) {
        chomp;
        $origline = $_;
        s/#.*$//;
        s/^\s+//;
        s/\s+$//;
        ( $comments .= "$origline\n" ), next if /^$/;

        undef $origkey;
        undef $key;
        undef $defaultvalue;
        $origkey = '';
        /^(.*?)\s*=/;
        $origkey = $1;
        /=\s*(.*)$/;
        $defaultvalue = $1;

        if ( defined($origkey) ) {
            $key = lc($origkey);
            if ( exists $oldsettings{$key} ) {
                print $keepcomments? $oldcomments{$key} : $comments;
                print "$origkey = $oldsettings{$key}\n";
                delete $oldsettings{$key};
                $comments = "";
                $UsedOldValue++;
            }
            else {

                # new value
                print $comments;
                print "$origline\n";

                # Indicate new value added
                print STDERR "Added new: $origline\n";
                sleep(2);
                $comments = "";
                $UsedDefaultValue++;
            }
        }
        else {

            # multi line values and other parts of manifest
            print "$origline\n";
        }
    }
    $newfh->close();
}

process_oldfile();
process_newfile();

while ( ( $key, $value ) = each %oldsettings ) {
    print STDERR "Removed old: $oldkeys{$key} = $value\n";
    sleep(2);
}

print STDERR <<EOL;

Summary
-------
Read $ReadOldValue settings from old manifest $oldconfig
Used $UsedOldValue settings from old manifest $oldconfig
Used $UsedDefaultValue default settings from new manifest $newconfig

EOL
exit 0;

__END__

=head1 update-puppet-config.pl
update-puppet-config.pl - Update Baruwa Puppet Manifest

=head1 SYNOPSIS
update-puppet-config.pl [oldconfig] [newconfig]

 Options:
    -oldconfig The existing Baruwa puppet manifest file
    -newconfig The new updated Baruwa puppet manifest file
    -help brief help message

=head1 OPTIONS

=over 8

=item B<-oldconfig>

Specify the existing Baruwa puppet manifest file name

=item B<-newconfig>

Specify the new puppet Baruwa manifest file name, usually init.pp

=item B<-help>

Print a brief help message and exits.

=back

=head1 DESCRIPTION

B<This program> will update the exiting Baruwa puppet manifest file
with options from a new manifest file.

=cut

